@@ -91,7 +91,8 @@ class TimeType(type_base.BaseType):
         self.__secs = fprime.common.models.serialize.numerical_types.U32Type(seconds)
         self.__usecs = fprime.common.models.serialize.numerical_types.U32Type(useconds)
 
-    def _check_useconds(self, useconds):
+    @staticmethod
+    def _check_useconds(useconds):
         """
         Checks if a given microsecond value is valid.
 
@@ -104,7 +105,8 @@ class TimeType(type_base.BaseType):
         if (useconds < 0) or (useconds > 999999):
             raise TypeRangeException(useconds)
 
-    def _check_time_base(self, time_base):
+    @staticmethod
+    def _check_time_base(time_base):
         """
         Checks if a given TimeBase value is valid.
 
@@ -286,13 +288,12 @@ class TimeType(type_base.BaseType):
         # If we could convert to a valid datetime, use that, otherwise, format
         if dt:
             return dt.strftime("%Y-%m-%d %H:%M:%S%z")
-        else:
-            return "%s: %d.%06ds, context=%d" % (
-                TimeBase(self.__timeBase.val).name,
-                self.__secs.val,
-                self.__usecs.val,
-                self.__timeContext.val,
-            )
+        return "%s: %d.%06ds, context=%d" % (
+            TimeBase(self.__timeBase.val).name,
+            self.__secs.val,
+            self.__usecs.val,
+            self.__timeContext.val,
+        )
 
     def get_datetime(self, tz=None):
         """
@@ -309,7 +310,7 @@ class TimeType(type_base.BaseType):
         tb = TimeBase(self.__timeBase.val)
         dt = None
 
-        if tb == TimeBase["TB_WORKSTATION_TIME"] or tb == TimeBase["TB_SC_TIME"]:
+        if tb in [TimeBase["TB_WORKSTATION_TIME"], TimeBase["TB_SC_TIME"]]:
 
             # This finds the local time corresponding to the timestamp and
             # timezone object, or local time zone if tz=None
@@ -353,8 +354,7 @@ class TimeType(type_base.BaseType):
         """Less than"""
         if isinstance(other, TimeType):
             return self.compare(self, other) < 0
-        else:
-            return self.get_float() < other
+        return self.get_float() < other
 
     def __le__(self, other):
         """Less than or equal"""
@@ -367,29 +367,25 @@ class TimeType(type_base.BaseType):
         """Equal"""
         if isinstance(other, TimeType):
             return self.compare(self, other) == 0
-        else:
-            return self.get_float() == other
+        return self.get_float() == other
 
     def __ne__(self, other):
         """Not equal"""
         if isinstance(other, TimeType):
             return self.compare(self, other) != 0
-        else:
-            return self.get_float() != other
+        return self.get_float() != other
 
     def __gt__(self, other):
         """Greater than"""
         if isinstance(other, TimeType):
             return self.compare(self, other) > 0
-        else:
-            return self.get_float() > other
+        return self.get_float() > other
 
     def __ge__(self, other):
         """Greater than or equal"""
         if isinstance(other, TimeType):
             return self.compare(self, other) >= 0
-        else:
-            return self.get_float() >= other
+        return self.get_float() >= other
 
     # The following helper methods enable support for arithmetic operations on TimeTypes.
 
