@@ -40,6 +40,7 @@ class CMakeHandler:
         self.settings = {}
         self._cmake_cache = None
         self.verbose = False
+        self.cached_help_targets = None
         try:
             self._run_cmake(["--help"], print_output=False)
         except Exception as exc:
@@ -54,13 +55,15 @@ class CMakeHandler:
     def validate_cmake_cache(self, cmake_args, build_dir):
         cmake_cache = self._read_cache(build_dir)
         for key, expected in cmake_args.items():
-            actual = cmake_cache.get(key, None)
+            actual = cmake_cache.get(key)
             # Allow expected variables not to be set in cache.
             # When building with a newer version of fprime-util and an older version of F prime,
             # newer CMake options that aren't used by older versions of CMake won't be set in the cache
             if actual is None:
                 continue
-            if str(expected) != actual:
+            expected = str(expected).strip()
+            actual = str(actual).strip()
+            if expected != actual:
                 raise CMakeInconsistentCacheException(key, expected, actual)
 
     def execute_known_target(
@@ -95,22 +98,20 @@ class CMakeHandler:
 
         make_args = {} if make_args is None else make_args
         fleshed_args = ["--"] + list(
-            map(lambda key: "{}={}".format(key, make_args[key]), make_args.keys())
+            map(lambda key: f"{key}={make_args[key]}", make_args.keys())
         )
 
         if target != "" and top_target:
             cmake_target = target
         else:
             module = self.get_cmake_module(path, build_dir)
-            cmake_target = (
-                module
-                if target == ""
-                else (
-                    "{}_{}".format(module, target).lstrip("_")
-                    if not top_target
-                    else target
-                )
-            )
+            if target == "":
+                cmake_target = module
+            elif top_target:
+                cmake_target = target
+            else:
+                cmake_target = f"{module}_{target}".lstrip("_")
+
         run_args = ["--build", build_dir]
         environment = {} if environment is None else copy.copy(environment)
         if self.verbose:
@@ -169,11 +170,12 @@ class CMakeHandler:
     def get_include_info(self, path, cmake_dir):
         """
         Calculates the include root of the given path. The include root is defined as the following based on the other
-        two values supplied. First, the following two path values are established:
+        two values supplied. First, the following paths values are established:
         - Location of the project's root. This is defined in the project_dir's CMakeList.txt, or in the CMake Cache.
         - Location of the project's F prime checkout. This is defined in the same places.
-        From there, the include root of the supplied path is whichever of those two paths is your parent. In cases where
-        both are parents, it will take the outer-most parent
+        - Locations of any libraries defined in the same places.
+        From there, the include root of the supplied path is whichever of those paths is your parent. In cases where
+        both are parents, it will take the nearest parent.
 
         :param path: path to calculate looking for include-root
         :param cmake_dir: directory of a CMake build, or directory containing a CMake project
@@ -265,7 +267,7 @@ class CMakeHandler:
             ),
             args.keys(),
         )
-        self._cmake_validate_source_dir(source_dir)
+        self.cmake_validate_source_dir(source_dir)
         self._run_cmake(
             ["-S", source_dir] + list(fleshed_args),
             workdir=build_dir,
@@ -319,21 +321,25 @@ class CMakeHandler:
             This code might not work on non-GNU makefile variants of CMake as it depends on the "help" target
 
         Returns:
-            list of CMake make targets
+            list of contextual CMake make targets
         """
-        run_args = ["--build", build_dir, "--target", "help"]
-        stdout, _ = self._run_cmake(run_args, write_override=True, print_output=False)
-        prefix = self.get_cmake_module(path, build_dir)
+        if not self.cached_help_targets:
+            run_args = ["--build", build_dir, "--target", "help"]
+            stdout, _ = self._run_cmake(
+                run_args, write_override=True, print_output=False
+            )
+            self.cached_help_targets = [
+                line.replace("...", "").strip()
+                for line in stdout
+                if line.startswith("...")
+            ]
 
-        make_target_names = [
-            line.replace("...", "").strip() for line in stdout if line.startswith("...")
-        ]
-        contextual_make_targets = [
+        prefix = self.get_cmake_module(path, build_dir)
+        return [
             make.replace(prefix, "").strip("_")
-            for make in make_target_names
+            for make in self.cached_help_targets
             if make.startswith(prefix)
         ]
-        return contextual_make_targets
 
     @staticmethod
     def purge(build_dir):
@@ -354,7 +360,7 @@ class CMakeHandler:
         """
         cache = self._read_cache(build_dir)
         # Reads cache values suppressing KeyError, {}.get(x, default=None)
-        miner = lambda x: cache.get(x, None)
+        miner = lambda x: cache.get(x)
         return tuple(map(miner, keys))
 
     def _read_cache(self, build_dir):
@@ -382,7 +388,7 @@ class CMakeHandler:
         return self._cmake_cache
 
     @staticmethod
-    def _cmake_validate_source_dir(source_dir):
+    def cmake_validate_source_dir(source_dir):
         """
         Raises an exception if the source dir is not a valid CMake source directory. This means a CMakeLists.txt exists
         and defines within it a project call.
@@ -393,13 +399,13 @@ class CMakeHandler:
         if not os.path.isfile(cmake_file):
             raise CMakeProjectException(source_dir, "No CMakeLists.txt is defined")
         # Test the cmake_file for project(
-        with open(cmake_file) as file_handle:
+        with open(cmake_file, encoding="utf8") as file_handle:
             project_lines = list(
                 filter(lambda line: "project(" in line, file_handle.readlines())
             )
             if not project_lines:
                 raise CMakeProjectException(
-                    source_dir, "No 'project()' calls in {}".format(cmake_file)
+                    source_dir, f"No 'project()' calls in {cmake_file}"
                 )
 
     @staticmethod
@@ -477,9 +483,9 @@ class CMakeHandler:
             cargs.append("-N")
         cargs.extend(arguments)
         if self.verbose:
-            print("[CMAKE] '{}'".format(" ".join(cargs)))
-            for key, val in cm_environ.items():
-                print("[CMAKE]     {}={}".format(key, val))
+            print(f"""[CMAKE] '{" ".join(cargs)}'""")
+            for key, val in sorted(cm_environ.items(), key=lambda x: x[0]):
+                print(f"[CMAKE]     {key}={val}")
 
         # In order to get proper console highlighting while still getting access to the output, we need to create a
         # pseudo-terminal. This will allow the proc to write to one side, and our select below to read from the other
@@ -504,9 +510,10 @@ class CMakeHandler:
         # needed. Thus we do not just exit.
         if ret != 0:
             raise CMakeExecutionException(
-                "CMake erred with return code {}".format(proc.returncode),
+                f"CMake erred with return code {proc.returncode}",
                 stderr,
                 print_output,
+                ret,
             )
         return stdout, stderr
 
@@ -550,7 +557,7 @@ class CMakeHandler:
                 appendable.append(line)
                 # Streams are EOF when the line returned is empty. Once this occurs, we are responsible for closing the
                 # stream and thus closing the select loop. Empty strings need not be printed.
-                if line == "":
+                if not line:
                     key.fileobj.close()
                     continue
                 # Forwards output to screen.  Assuming a PTY is used, then coloring highlights should be automatically
@@ -567,6 +574,10 @@ class CMakeHandler:
 class CMakeException(FprimeException):
     """Error occurred within this CMake package"""
 
+    def __init__(self, message, exit_code=1):
+        super().__init__(message)
+        self.exit_code = exit_code
+
 
 class CMakeInconsistentCacheException(CMakeException):
     """Project CMakeCache.txt is inconsistent with settings.ini file"""
@@ -574,9 +585,7 @@ class CMakeInconsistentCacheException(CMakeException):
     def __init__(self, key, expected, actual):
         """Force an appropriate message"""
         super().__init__(
-            "Expected CMake variable {} to be set to '{}', was actually set to '{}'. This is usually caused by updating the settings.ini file without purging and regenerating the accompanying build cache.".format(
-                key, expected, actual
-            )
+            f"Expected CMake variable {key} to be set to '{expected}', was actually set to '{actual}'. This is usually caused by updating the settings.ini file without purging and regenerating the accompanying build cache."
         )
 
 
@@ -585,7 +594,7 @@ class CMakeOrphanException(CMakeException):
 
     def __init__(self, file_dir):
         """Force an appropriate message"""
-        super().__init__("{} is outside the F prime project".format(file_dir))
+        super().__init__(f"{file_dir} is outside the F prime project")
 
 
 class CMakeProjectException(CMakeException):
@@ -593,9 +602,7 @@ class CMakeProjectException(CMakeException):
 
     def __init__(self, project_dir, error):
         """Force an appropriate message"""
-        super().__init__(
-            "{} is an invalid F prime deployment. {}".format(project_dir, error)
-        )
+        super().__init__(f"{project_dir} is an invalid F prime deployment. {error}")
 
 
 class CMakeInvalidBuildException(CMakeException):
@@ -604,18 +611,16 @@ class CMakeInvalidBuildException(CMakeException):
     def __init__(self, build_dir):
         """Force an appropriate message"""
         super().__init__(
-            "{} is not a CMake build directory. Please setup using 'fprime-util generate'".format(
-                build_dir
-            )
+            f"{build_dir} is not a CMake build directory. Please setup using 'fprime-util generate'"
         )
 
 
 class CMakeExecutionException(CMakeException):
     """Pass up a CMake Error as an Exception"""
 
-    def __init__(self, message, stderr, printed):
+    def __init__(self, message, stderr, printed, exit_code=1):
         """The error data should be stored"""
-        super().__init__(message)
+        super().__init__(message, exit_code)
         self.stderr = stderr
         self.need = not printed
 
@@ -639,4 +644,4 @@ class CMakeNoSuchTargetException(CMakeException):
 
     def __init__(self, build_dir, target):
         """Better messaging for this exception"""
-        super().__init__("{} does not support target {}".format(build_dir, target))
+        super().__init__(f"{build_dir} does not support target {target}")
