@@ -13,7 +13,8 @@ from cookiecutter.main import cookiecutter
 from cookiecutter.exceptions import OutputDirExistsException
 from jinja2 import Environment, FileSystemLoader
 
-from fprime.fbuild.builder import Build, Target
+from fprime.fbuild.builder import Build
+from fprime.fbuild.target import Target
 from fprime.fbuild.cmake import CMakeExecutionException
 
 
@@ -26,19 +27,15 @@ def confirm(msg):
             return True
         if confirm_input.lower() in ["n", "no"]:
             return False
-        print("{} is invalid.  Please use 'yes' or 'no'".format(confirm_input))
+        print(f"{confirm_input} is invalid.  Please use 'yes' or 'no'")
 
 
 def replace_contents(filename, what, replacement, count=1):
-    with open(filename) as fh:
-        changelog = fh.read()
+    changelog = Path(filename).read_text()
     with open(filename, "w") as fh:
         new_file = changelog.replace(what, replacement, count)
         fh.write(new_file)
-        if new_file != changelog:
-            return True
-        else:
-            return False
+        return new_file != changelog
 
 
 def run_impl(deployment: Path, path: Path, platform: str, verbose: bool):
@@ -47,8 +44,8 @@ def run_impl(deployment: Path, path: Path, platform: str, verbose: bool):
     build = Build(target.build_type, deployment, verbose=verbose)
     build.load(platform)
 
-    hpp_files = glob.glob("{}/*.hpp".format(path), recursive=False)
-    cpp_files = glob.glob("{}/*.cpp".format(path), recursive=False)
+    hpp_files = glob.glob(f"{path}/*.hpp", recursive=False)
+    cpp_files = glob.glob(f"{path}/*.cpp", recursive=False)
     cpp_files.sort(key=len)
 
     # Check destinations
@@ -63,15 +60,15 @@ def run_impl(deployment: Path, path: Path, platform: str, verbose: bool):
     cpp_dest = common[0] if common else cpp_files[0]
 
     if not confirm(
-        "Generate implementations and merge into {} and {}?".format(hpp_dest, cpp_dest)
+        f"Generate implementations and merge into {hpp_dest} and {cpp_dest}?"
     ):
         return False
     print("Generating implementation files and merging...")
     with suppress_stdout():
         build.execute(target, context=path, make_args={})
 
-    hpp_files_template = glob.glob("{}/*.hpp-template".format(path), recursive=False)
-    cpp_files_template = glob.glob("{}/*.cpp-template".format(path), recursive=False)
+    hpp_files_template = glob.glob(f"{path}/*.hpp-template", recursive=False)
+    cpp_files_template = glob.glob(f"{path}/*.cpp-template", recursive=False)
 
     if not hpp_files_template or not cpp_files_template:
         print("[WARNING] Failed to find generated .cpp-template or .hpp-template files")
@@ -96,7 +93,7 @@ def run_impl(deployment: Path, path: Path, platform: str, verbose: bool):
 
 def add_to_cmake(list_file: Path, comp_path: Path):
     """Adds new component or port to CMakeLists.txt"""
-    print("[INFO] Found CMakeLists.txt at '{}'".format(list_file))
+    print(f"[INFO] Found CMakeLists.txt at '{list_file}'")
     with open(list_file, "r") as f:
         lines = f.readlines()
 
@@ -107,9 +104,7 @@ def add_to_cmake(list_file: Path, comp_path: Path):
         print("Already added to CMakeLists.txt")
         return True
 
-    if not confirm(
-        "Add component {} to {} {}".format(comp_path, list_file, "at end of file?")
-    ):
+    if not confirm(f"Add component {comp_path} to {list_file} at end of file?"):
         return False
 
     lines.append(addition)
@@ -188,7 +183,7 @@ def add_unit_tests(deployment, comp_path, platform, verbose):
 
 def add_port_to_cmake(list_file: Path, comp_path: Path):
     """Adds new port to CMakeLists.txt in port directory"""
-    print("[INFO] Found CMakeLists.txt at '{}'".format(list_file))
+    print(f"[INFO] Found CMakeLists.txt at '{list_file}'")
     with open(list_file, "r") as file_handle:
         lines = file_handle.readlines()
     index = 0
@@ -197,9 +192,7 @@ def add_port_to_cmake(list_file: Path, comp_path: Path):
     index += 1
     while "CMAKE_CURRENT_LIST_DIR" in lines[index]:
         index += 1
-    if not confirm(
-        "Add port {} to {} {}?".format(comp_path, list_file, "ports in CMakeLists.txt")
-    ):
+    if not confirm(f"Add port {comp_path} to {list_file} ports in CMakeLists.txt?"):
         return False
 
     addition = '    "${{CMAKE_CURRENT_LIST_DIR}}/{}"\n'.format(comp_path)
@@ -253,7 +246,7 @@ def new_component(deployment: Path, platform: str, verbose: bool, build: Build):
             and build.get_settings("component_cookiecutter", None) != "default"
         ):
             source = build.get_settings("component_cookiecutter", None)
-            print("[INFO] Cookiecutter source: {}".format(source))
+            print(f"[INFO] Cookiecutter source: {source}")
         else:
             source = (
                 os.path.dirname(__file__)
@@ -266,9 +259,7 @@ def new_component(deployment: Path, platform: str, verbose: bool, build: Build):
         ).resolve()
         if proj_root is None:
             print(
-                "[INFO] Created component directory without adding to build system nor generating implementation {}".format(
-                    final_component_dir
-                )
+                f"[INFO] Created component directory without adding to build system nor generating implementation {final_component_dir}"
             )
             return 0
         # Attempt to register to CMakeLists.txt
@@ -280,9 +271,7 @@ def new_component(deployment: Path, platform: str, verbose: bool, build: Build):
             cmake_lists_file, final_component_dir.relative_to(cmake_lists_file.parent)
         ):
             print(
-                "[INFO] Could not register {} with build system. Please add it and generate implementations manually.".format(
-                    final_component_dir
-                )
+                f"[INFO] Could not register {final_component_dir} with build system. Please add it and generate implementations manually."
             )
             return 0
         regenerate(build)
@@ -303,17 +292,15 @@ def new_component(deployment: Path, platform: str, verbose: bool, build: Build):
 
         add_unit_tests(deployment, final_component_dir, platform, verbose)
         print(
-            "[INFO] Next run `fprime-util build{}` in {}".format(
-                "" if platform is None else " " + platform, final_component_dir
-            )
+            f'[INFO] Next run `fprime-util build{"" if platform is None else f" {platform}"}` in {final_component_dir}'
         )
         return 0
     except OutputDirExistsException as out_directory_error:
-        print("{}".format(out_directory_error), file=sys.stderr)
+        print(f"{out_directory_error}", file=sys.stderr)
     except CMakeExecutionException as exc:
-        print("[ERROR] Failed to create component. {}".format(exc), file=sys.stderr)
+        print(f"[ERROR] Failed to create component. {exc}", file=sys.stderr)
     except OSError as ose:
-        print("[ERROR] {}".format(ose))
+        print(f"[ERROR] {ose}")
     return 1
 
 
@@ -345,7 +332,7 @@ def is_valid_name(word: str):
     for char in invalid_characters:
         if isinstance(word, str) and char in word:
             return char
-        elif not isinstance(word, str):
+        if not isinstance(word, str):
             raise ValueError("Incorrect usage of is_valid_name")
     return "valid"
 
@@ -373,17 +360,16 @@ def get_port_input(namespace):
     }
     args_done = False
     arg_list = []
-    port_name = get_valid_input("Port Name [{}]: ".format(defaults["port_name"]))
-    short_description = input(
-        "Short Description [{}]: ".format(defaults["short_description"])
-    )
-    dir_name = get_valid_input("Directory Name [{}]: ".format(defaults["dir_name"]))
-    namespace = get_valid_input("Port Namespace [{}]: ".format(defaults["namespace"]))
+    port_name = get_valid_input(f'Port Name [{defaults["port_name"]}]: ')
+    short_description = input(f'Short Description [{defaults["short_description"]}]: ')
+    dir_name = get_valid_input(f'Directory Name [{defaults["dir_name"]}]: ')
+    namespace = get_valid_input(f'Port Namespace [{defaults["namespace"]}]: ')
     while not args_done:
-        if arg_list == []:
-            add_arg = confirm("Would you like to add an argument?: ")
-        else:
-            add_arg = confirm("Would you like to add another argument?: ")
+        add_arg = (
+            confirm("Would you like to add another argument?: ")
+            if arg_list
+            else confirm("Would you like to add an argument?: ")
+        )
         if add_arg:
             arg_name = get_valid_input("Argument name: ")
             arg_type = get_valid_input(
@@ -459,7 +445,7 @@ def new_port(deployment: Path, build: Build):
         else:
             add_port_to_cmake(str(path_to_cmakelists), fname)
 
-        if proj_root_found is False:
+        if not proj_root_found:
             print(
                 "[INFO] No project root found. Created port without adding to build system nor generating implementation."
             )
@@ -472,9 +458,7 @@ def new_port(deployment: Path, build: Build):
             (Path(context["dir_name"]).resolve()).relative_to(cmake_lists_file.parent),
         ):
             print(
-                "[INFO] Could not register {} with build system. Please add it and generate implementations manually.".format(
-                    Path(context["dir_name"]).resolve()
-                )
+                f'[INFO] Could not register {Path(context["dir_name"]).resolve()} with build system. Please add it and generate implementations manually.'
             )
             return 0
         regenerate(build)
@@ -495,9 +479,9 @@ def new_port(deployment: Path, build: Build):
         )
         return 0
     except OutputDirExistsException as out_directory_error:
-        print("{}".format(out_directory_error), file=sys.stderr)
+        print(f"{out_directory_error}", file=sys.stderr)
     except CMakeExecutionException as exc:
-        print("[ERROR] Failed to create port. {}".format(exc), file=sys.stderr)
+        print(f"[ERROR] Failed to create port. {exc}", file=sys.stderr)
     except OSError as ose:
-        print("[ERROR] {}".format(ose))
+        print(f"[ERROR] {ose}")
     return 1
